FusionHUD = LibStub("AceAddon-3.0"):NewAddon("FusionHUD", "AceConsole-3.0");
local AceGUI = LibStub("AceGUI-3.0");

local _G = _G;

local FusionHUDBarPrototype = {};
FusionHUDBarPrototype.__index = FusionHUDBarPrototype;

local FusionHUDLockFrame = CreateFrame("Frame", nil, UIParent);

local FusionHUDBarType = { 
	"Health", 
	"Power" 
}

local FusionHUDLastUpdateClock = GetTime();
 

local FusionHUDBars = {};
local FusionHUDModSettings = {};

FusionHUDOptions = {
    name = "Fusion HUD",
    handler = FusionHUD,
    type = 'group',
    args = {
		mainOptions = {
			name = "Main Menu",
			order = 1,
			type = "group",
			desc = "Main menu for FusionHUD",
			args = {
				newBar = {
					name = "New Bar",
					type = "execute",
					desc = "Creates a new HUD bar",
					order = 1,
					func = 		function()
									local index = 0;
									table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
									FusionHUDBars["bar" .. index + 1] = FusionHUDBarPrototype:New(index + 1)
								end,
				},
				lockAll = {
					name = "Unlock All",
					type = "execute",
					desc = "Unlocks all HUD bars for repositioning",
					order = 2,
					func = 	function() 
								table.foreach(FusionHUDBars, 	function(k,v)
																	v.draggable = true;
																	v:LockHandler();
																end);
								FusionHUDLockFrame:Show();
								InterfaceOptionsFrame:Hide();
							end,
				},
				spacer1 = {
					name = "___________________________",
					type = "header",
					order = 99,
				},	
				deleteBar = {
					name = "Delete Bar",
					order = 100,
					type = "execute",
					desc = "Deletes the last bar created",
					confirm = function() return "Are you Sure? This cannot be undone" end,
					func =	function()
								FusionHUD:DeleteBar() 
							end	
				},
			}
		},
		bars = {
			name = "Bar List",
			order = 2,
			type = "group",
			desc = "Listing of each HUD bar",
			args = {}
		},
    },
};

function FusionHUD:OnInitialize()	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FusionHUD", FusionHUDOptions);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FusionHUD", "FusionHUD");
	
	FusionHUDMasterFrame = CreateFrame("Frame", nil, UIParent);
	
	FusionHUDMasterFrame:SetClampedToScreen(true);
	FusionHUDMasterFrame:SetPoint("CENTER",UIParent,"CENTER");
	FusionHUDMasterFrame:SetFrameStrata("LOW");
	FusionHUDMasterFrame:SetHeight(400);
	FusionHUDMasterFrame:SetWidth(400);
	FusionHUDMasterFrame:Show();
	FusionHUDMasterFrame:SetAlpha(1);

	FusionHUDLockFrame:SetClampedToScreen(true);
	FusionHUDLockFrame:ClearAllPoints();
	FusionHUDLockFrame:SetPoint("TOP", UIParent, "TOP", 0, -50);
	FusionHUDLockFrame:SetFrameStrata("FULLSCREEN");
	FusionHUDLockFrame:SetHeight(100);
	FusionHUDLockFrame:SetWidth(200);
	FusionHUDLockFrame.texture = FusionHUDLockFrame:CreateTexture(nil, "BACKGROUND");
	FusionHUDLockFrame.texture:SetAllPoints(FusionHUDLockFrame);
	FusionHUDLockFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDLockFrame.texture:SetVertexColor(0.1, 0.1, 0.1, 0.75);
	FusionHUDLockFrame.texture:SetTexCoord(0,1,0,1)
	FusionHUDLockFrame.text = FusionHUDLockFrame:CreateFontString(nil, "LOW")
	FusionHUDLockFrame.text:SetPoint("TOP", FusionHUDLockFrame, "TOP",0, -20);
	FusionHUDLockFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 12, "BOLD");
	FusionHUDLockFrame.text:SetJustifyH("CENTER");
	FusionHUDLockFrame.text:SetJustifyV("TOP");
	FusionHUDLockFrame.text:SetHeight(30);
	FusionHUDLockFrame.text:SetWidth(150);
	FusionHUDLockFrame.text:SetText("FusionHUD");	
	
	FusionHUDLockFrame.lockButton = CreateFrame("Button", "lockButton", FusionHUDLockFrame, "UIPanelButtonTemplate")
	FusionHUDLockFrame.lockButton:SetPoint("BOTTOM", FusionHUDLockFrame, "BOTTOM", 0, 25);
	FusionHUDLockFrame.lockButton:SetWidth(80);
	FusionHUDLockFrame.lockButton:SetHeight(25);
	FusionHUDLockFrame.lockButton:SetText("Lock");
	FusionHUDLockFrame.lockButton:SetFrameStrata("FULLSCREEN_DIALOG");
	FusionHUDLockFrame.lockButton:SetScript('OnClick', 	function() 
															table.foreach(FusionHUDBars,	function(k,v) 
																								FusionHUDBars[k].draggable = false;
																								FusionHUDBars[k]:LockHandler();
																							end) 
														end);
	FusionHUDLockFrame:Hide();
	
	FusionHUDMasterFrame:SetScript("OnUpdate", 	function() FusionHUD:OnUpdate()	end); 	
end

function FusionHUD:OnEnable()
	local name = UnitName("player");
	if FusionHUDDB and FusionHUDDB[name] then
		FusionHUDBars = {};
		table.foreach(FusionHUDDB[name].bars,	function(k,v)
											FusionHUDBars[k] = FusionHUDDB[name].bars[k];
											setmetatable(FusionHUDBars[k], FusionHUDBarPrototype);
											FusionHUDBars[k].lastValue = -99999999999;
											FusionHUDBars[k].lastValueMax = -99999999999;
											FusionHUDBars[k]:BuildBar();
										end);
		FusionHUDDB[name].bars = FusionHUDBars;
		FusionHUDDB[name].settings = FusionHUDModSettings;
	elseif FusionHUDDB then
		FusionHUDDB[name] = {};
		FusionHUDDB[name].settings = FusionHUDModSettings;
		FusionHUDDB[name].bars = FusionHUDBars;
	else
		FusionHUDDB = {};
		FusionHUDDB[name] = {};
		FusionHUDDB[name].settings = FusionHUDModSettings;
		FusionHUDDB[name].bars = FusionHUDBars;
		
	end
end	

function FusionHUDBarPrototype:New(index)		--New Bar object, initializes the frames and textures
	local self = {};
    setmetatable(self, FusionHUDBarPrototype); -- Set the metatable so we used FusionHUDBarPrototype's __index
	
	self.barIndex = index;
	self.lastHeight = 400;
	self.lastWidth = 15;
	self.draggable = false;
	self.parentFrame = "FusionHUDMasterFrame";
	self.myPoint = "CENTER";
	self.parentPoint = "CENTER";
	self.offsetX = 0;
	self.offsetY = 0;
	self.orientation = "LEFT";
	self.textureName = "Interface\\AddOns\\FusionHUD\\Graphics\\White Straight Bar Base";
	self.containerTextureName = "Interface\\AddOns\\FusionHUD\\Graphics\\Straight Bar Glass Container";
	self.height = 400;
	self.width = 15;
	self.barType = "Health";
	self.barTarget = "player";
	self.totals = true;
	self.text = "EEEEEEE";
	self.warningColor = false;
	self.lastValue = -999999999999999;
	self.lastValueMax = -9999999999999999;

	self:BuildBar();
	
	return self;
end

function FusionHUDBarPrototype:LockHandler()
	if self.draggable == true then
		self.frame.titleRegion:SetAllPoints(self.frame);
		self.frame:SetBackdrop({bgFile = "Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", tile = true, tilesize = self.width});
		self.frame:SetBackdropColor(0.6, 0.6, 0.6, 0.75);
		self.frame:Show();
		self.frame:SetAlpha(1);
		self.frame:EnableMouse(true);
		self.frame:SetMovable(true);
		return
	else
		self.myPoint, self.parentFrame, self.parentPoint, self.offsetX, self.offsetY = self.frame:GetPoint();
		self.frame.titleRegion:ClearAllPoints();
		self.frame:SetBackdrop({nil});
		self.frame:EnableMouse(false);
		self.frame:SetMovable(false);
--		self.frame.dragTexture:ClearAllPoints()
		if FusionHUDLockFrame:IsShown() then
			FusionHUDLockFrame:Hide();
			InterfaceOptionsFrame:Show();
		end
	end
end

function FusionHUDBarPrototype:Update()	-- The bar update function
	if UnitName(self.barTarget) == nil then
		self.frame:Hide();
		return
	else
		self.frame:Show();
		if self.barType == FusionHUDBarType[1] then  --check for type of bar, power or health
			-- check to see if dimension or health/power values have changed
			if self.lastValue ~= UnitHealth(self.barTarget) or self.lastValueMax ~= UnitHealthMax(self.barTarget) or self.height ~= self.lastHeight or self.width ~= self.lastWidth then
				self.lastHeight, self.lastWidth, self.lastValue, self.lastValueMax = self.height, self.width, UnitHealth(self.barTarget), UnitHealthMax(self.barTarget);
				--set color for the health bar
				self.frame.barTexture:SetVertexColor(0,1,0,1);
				self.frame.text:SetVertexColor(0,1,0,1);
				--set bar height and text for a health bar
				local health = UnitHealth(self.barTarget) .. "/" .. UnitHealthMax(self.barTarget);
				if (self.height *UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget)) == 0 then
					self.frame.barTexture:SetHeight(1);
				else
					self.frame.barTexture:SetHeight(self.height*UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget));
				end
				if UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget) >= 0 then
					self.frame.barTexture:SetTexCoord(0,1,UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget),0);
				else
					self.frame.barTexture:SetTexCoord(0,1,0,0);
				end
				
				self.frame.text:SetText(health);
			end
		elseif self.barType == FusionHUDBarType[2] then
			local powerType = UnitPowerType(self.barTarget);
			local power = UnitPower(self.barTarget) .. "/" .. UnitPowerMax(self.barTarget)
			-- check to see if values have changed
			if self.lastValue ~= UnitPower(self.barTarget) or self.lastValueMax ~= UnitPowerMax(self.barTarget) then
				self.lastValue = UnitPower(self.barTarget);
				self.lastValueMax = UnitPowerMax(self.barTarget);
				--set color for the power bar
				if powerType == 1 then
					self.frame.barTexture:SetVertexColor(0.8,0,0,1);
					self.frame.text:SetVertexColor(0.8,0,0,1);
				elseif powerType == 6 then
					self.frame.barTexture:SetVertexColor(0.6,0,0,1);
					self.frame.text:SetVertexColor(0.6,0,0,1);
				elseif powerType == 0 then
					self.frame.barTexture:SetVertexColor(0,0,0.8,1);
					self.frame.text:SetVertexColor(0,0,0.8,1);
				elseif powerType == 2 then
					self.frame.barTexture:SetVertexColor(0.8,0.3,0,1);
					self.frame.text:SetVertexColor(0.8,0.3,0,1);
				else
					self.frame.barTexture:SetVertexColor(1,1,0,1);
					self.frame.text:SetVertexColor(1,1,0,1);
				end
				--set bar height and text for the power bar
				if (self.height*UnitPower(self.barTarget)/UnitPowerMax(self.barTarget)) == 0 then
					self.frame.barTexture:SetHeight(1);
				else
					self.frame.barTexture:SetHeight(self.height*UnitPower(self.barTarget)/UnitPowerMax(self.barTarget));
				end
				if UnitPower(self.barTarget)/UnitPowerMax(self.barTarget) >= 0 then
					self.frame.barTexture:SetTexCoord(0,1,UnitPower(self.barTarget)/UnitPowerMax(self.barTarget),0);
				else
					self.frame.barTexture:SetTexCoord(0,1,0,0);
				end
				self.frame.text:SetText(power);
			end
		end
	end	
end

function FusionHUDBarPrototype:BuildBar()
	self.frame = nil;
	--Set up bar frame
	self.frame = CreateFrame("Frame", nil, FusionHUDMasterFrame);
	self.frame:ClearAllPoints();
	self.frame:SetPoint(self.myPoint, FusionHUDMasterFrame, self.parentPoint);
	self.frame:SetHeight(self.height);
	self.frame:SetWidth(self.width);
	self.frame:SetClampedToScreen(true);
	--Set Up the title region for the bar to enable dragging
	self.frame.titleRegion = self.frame:CreateTitleRegion();
	self.frame.titleRegion:ClearAllPoints();
	--Set up the bar texture
	self.frame.barTexture = self.frame:CreateTexture(nil, "BACKGROUND");
	self.frame.barTexture:SetTexture(self.textureName, false);
	self.frame.barTexture:ClearAllPoints();
	self.frame.barTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.barTexture:SetHeight(self.height);
	self.frame.barTexture:SetWidth(self.width);
	--Set up the bar background/scale texture
	self.frame.containerTexture = self.frame:CreateTexture(nil, "OVERLAY");
	self.frame.containerTexture:SetTexture(self.containerTextureName, false);
	self.frame.containerTexture:ClearAllPoints();
	self.frame.containerTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.containerTexture:SetHeight(self.height);
	self.frame.containerTexture:SetWidth(self.width);
	self.frame.containerTexture:SetTexCoord(0,1,1,0);
	self.frame.containerTexture:SetBlendMode("BLEND");
	--Set up bar text
	self.frame.text = self.frame:CreateFontString(nil, "LOW");
	self.frame.text:ClearAllPoints();
	self.frame.text:SetPoint("TOP", self.frame, "BOTTOM");
	self.frame.text:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE");
	self.frame.text:SetJustifyH("CENTER");
	self.frame.text:SetJustifyV("TOP");
	self.frame.text:SetHeight(30);
	self.frame.text:SetWidth(150);
	
	--Set Up the draggable frame
--	self.frame.dragTexture = self.frame:CreateTexture(nil, "OVERLAY");
--	self.frame.dragTexture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
--	self.frame.dragTexture:ClearAllPoints();
	
	self.frame:SetPoint(self.myPoint, self.parentFrame, self.parentPoint, self.offsetX, self.offsetY);
	
	--Set Up the bar's options table
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex] = {};
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].name = "Bar " .. self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].type = "group";
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].order = self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].args = {
		header = {
			name = "Bar " .. self.barIndex,
			order = 1,
			type = "header"
		},		
		lock = {
			name = "Locked",
			order = 2,
			type = "toggle",
			desc = "Unlocks the bar for repositioning",
			get = 	function() return not FusionHUDBars["bar" .. self.barIndex].draggable end;
			set = 	function() 
						self.draggable = true;
						self:LockHandler()
						FusionHUDLockFrame:Show();
						InterfaceOptionsFrame:Hide();
--						self.dragTexture:SetAllPoints(self.frame);
					end;
		},
		barType = {
			name = "Bar Type",
			order = 4,
			type = "select",
			desc = "The bar can track either Health or Power(Mana/Runic Power/Energy/Focus)",
			style = "dropdown",
			values = {
				Power = "Power", 
				Health = "Health"
			},
			get = 	function() return FusionHUDBars["bar" .. self.barIndex].barType end,
			set = 	function(key, value) FusionHUDBars["bar" .. self.barIndex].barType = value end
		},
		barTarget = {
			name = "Bar Target",
			order = 3,
			type = "select",
			desc = "The unit that the bar will track",
			style = "dropdown",
			values = {
				player = "Player",
				pet = "Pet",
				playertarget = "Player target",
				playertargettarget = "Target of Target",
				pettarget = "Pet Target"
			},
			get = 	function() return self.barTarget end,
			set = 	function(key, value) self.barTarget = value end
		},
		barHeight = {
			name = "Bar Height",
			order = 5,
			type = "range",
			min = 50,
			max = 1200,
			step = 1,
			get = 	function() return self.height end,
			set = 	function(k,v) 
						self.height = v; 
						self.frame:SetHeight(self.height); 
						self.frame.containerTexture:SetHeight(self.height);
						self.frame.barTexture:SetHeight(self.height);
					end
		},
		barWidth = {
			name = "Bar Width",
			order = 6,
			type = "range",
			min = 1,
			max = 100,
			step = 1,
			get = 	function() return self.width end,
			set = 	function(k,v) 
						self.width = v; 
						self.frame:SetWidth(self.width); 
						self.frame.containerTexture:SetWidth(self.width);
						self.frame.barTexture:SetWidth(self.width);
					end
		}
	}
end

function FusionHUD:DeleteBar()
	local index = -1;
	table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
	if index then
		FusionHUDOptions.args.bars.args["bar" .. index] = nil;
		FusionHUDBars["bar" .. index].frame:Hide();
		FusionHUDBars["bar" .. index] = nil;
	else
		FusionHUDBars = {};
	end
end

function FusionHUD:OnUpdate()
	local FusionHUDNextUpdateClock = GetTime()
	if FusionHUDNextUpdateClock - FusionHUDLastUpdateClock > 0.1 then
		FusionHUDLastUpdateClock = FusionHUDNextUpdateClock;
		table.foreach(FusionHUDBars, 	function(k,v) 
											FusionHUDBars[k]:Update() 
										end);
	end
end